import express from "express"
import fs from "node:fs/promises"
import path from "node:path"
import crypto from "node:crypto"
import ErrorAgregator from "../lib/errorAgregator.js";

const router = express.Router();

/*
RES
[
    {
        id: int,
        name: string,
        pic_url: string,
        update_frequency_seconds: int
        min_threshold: float,
        max_threshold: float,
        last_data_point_date: date,
        last_data_point_value: float
    },
    {...}
]
 */
router.get("/devices", async (req, res) => {
    const db = req.app.get("db");

    const devicesResult = await db.query(`
        SELECT 
            devices.id,
            devices.name, 
            devices.pic_url, 
            devices.min_threshold, 
            devices.max_threshold,
            last_data_point.point_transmited_datetime as "last_data_point_date",
            last_data_point.value as "last_data_point_value" 

        FROM 
            devices 
            
        LEFT JOIN (
            SELECT data_points.device_id, data_points.point_transmited_datetime, data_points.value
            FROM data_points
            WHERE id IN (
                SELECT MAX(id)
                FROM data_points
                GROUP BY device_id
            )
        ) as last_data_point

        ON
        last_data_point.device_id = devices.id
    `);

    if (Array.isArray(devicesResult) === false) {
        devicesResult = [devicesResult];
    }

    res.send(devicesResult);
});


/*
RES
{
    id: int,
    pic_url: string,
    update_frequency_seconds: int
    min_threshold: float,
    max_threshold: float
}
*/
router.get("/devices/:id", async (req, res) => {
    const db = req.app.get("db");

    const deviceId = Number.parseInt(req.params.id);

    if (isNaN(deviceId)) {
        res.send({
            error: "Id must be a number"
        })
        return;
    }

    const deviceResult = await db.query(`
        SELECT 
            devices.id,
            devices.name, 
            devices.pic_url, 
            devices.min_threshold, 
            devices.max_threshold,
            update_frequency_seconds

        FROM 
            devices 

            
        LEFT JOIN 
            data_points  


        ON
            data_points.device_id = devices.id

        WHERE
            devices.id = ${deviceId}


        GROUP BY
            devices.id
    `);

    if (deviceResult === null || deviceResult === undefined || deviceResult.length === 0) {
        res.status(404);
        res.send({
            error: "Device not found"
        })
        return;
    }

    res.send(deviceResult[0]);
})


/*
REQ
{
    name: string,
    base64img: string,
    update_frequency_seconds: int
    min_threshold: float,
    max_threshold: float
}
*/
/*
RES
{
    id: int
}
 */
router.post("/devices", async (req, res) => {
    const db = req.app.get("db");

    const finalValues = {
        ...{
            name: null,
            pic_url: null,
            update_frequency_seconds: 3600,
            min_threshold: 20,
            max_threshold: 80
        },
        ...req.body
    }

    const errorAgregator = new ErrorAgregator();

    if (req.body.name == null || req.body.name.trim() === "") {
        errorAgregator.pushError("name", "name can't be null or empty string");
    }

    if (req.body.update_frequency_seconds != null && (Number.isNaN(Number.parseInt(req.body.update_frequency_seconds)) || Number.parseInt(req.body.update_frequency_seconds) <= 0)) {
        errorAgregator.pushError("update_frequency_seconds", "update_frequency_seconds must be a number > 0");
    }

    if (finalValues.min_threshold === null || (finalValues.min_threshold !== null && (Number.isNaN(Number.parseInt(finalValues.min_threshold)) || Number.parseInt(finalValues.min_threshold) < 0))) {
        errorAgregator.pushError("min_threshold", "Minimum threshold must be a number >=0");
    }

    if (finalValues.max_threshold === null || (finalValues.max_threshold !== null && (Number.isNaN(Number.parseInt(finalValues.max_threshold)) || Number.parseInt(finalValues.max_threshold) > 100))) {
        errorAgregator.pushError("max_threshold", "Maximum threshold must be a number <=100");
    }

    if (
        (finalValues.max_threshold !== null && finalValues.max_threshold !== undefined || finalValues.min_threshold !== null && finalValues.min_threshold !== undefined)
        && Number.parseInt(finalValues.min_threshold) > Number.parseInt(finalValues.max_threshold)
    ) {
        errorAgregator.pushError("max_threshold", `Maximum threshold (${finalValues.max_threshold}) must be greater than minimum threshold (${finalValues.min_threshold})`);
    }

    if (errorAgregator.errors.length > 0) {
        res.status(400);
        res.send(errorAgregator.errors);
        return;
    }

    if (finalValues.base64img != null) {
        try {
            const fileData = Buffer.from(finalValues.base64img, "base64url");
            const fileUUID = crypto.randomUUID();
            const filePath = path.join(process.cwd(), `/static/img/${fileUUID}.jpeg`);

            await fs.writeFile(filePath, fileData);

            finalValues.pic_url = `/img/${fileUUID}.jpeg`;
        }
        catch (exc) {
            res.send({
                error: "Error saving image"
            });
            console.log(exc);
            return;
        }
    }

    const insertResult = await db.query(`
        INSERT INTO
            devices (name, pic_url, update_frequency_seconds, min_threshold, max_threshold)
        
        VALUES
            (?, ?, ?, ?, ?)
    `, [
        finalValues.name,
        finalValues.pic_url,
        finalValues.update_frequency_seconds,
        finalValues.min_threshold,
        finalValues.max_threshold
    ]);

    res.status(201);
    res.send({ id: Number.parseInt(insertResult.insertId) });
});


/*
REQ
{
    name: string,
    device_uuid: string,
    base64img: string,
    update_frequency_seconds: int
    min_threshold: float,
    max_threshold: float
}
*/
/*
RES
{
    id: int,
    name: string,
    pic_url: string,
    min_threshold: float,
    max_threshold: float,
    update_frequency_seconds: int,
    last_data_point_date: date,
    last_data_point_value: float
}
 */
router.patch("/devices/:id", async (req, res) => {
    const db = req.app.get("db");

    const deviceId = Number.parseInt(req.params.id);

    if (isNaN(deviceId)) {
        res.send({
            error: "Id must be a number"
        })
        return;
    }

    const deviceResult = await db.query(` 
        SELECT 
        devices.id

        FROM 
            devices 

        WHERE
            devices.id = ${deviceId}


        GROUP BY
            devices.id
    `);

    if (deviceResult === null || deviceResult === undefined || deviceResult.length === 0) {
        res.status(404);
        res.send({
            error: "Device not found"
        })
        return;
    }

    //if no new data was sent, return the current un-updated device
    if (req.body === undefined || req.body === null || Object.getOwnPropertyNames(req.body).length === 0) {
        const currentDevice = await db.query(`
            SELECT 
            devices.id,
            devices.name, 
            devices.pic_url, 
            devices.min_threshold, 
            devices.max_threshold,
            last_data_point.point_transmited_datetime as "last_data_point_date",
            last_data_point.value as "last_data_point_value" 

            FROM 
                devices 
                
            LEFT JOIN (
                SELECT data_points.device_id, data_points.point_transmited_datetime, data_points.value
                FROM data_points
                WHERE id IN (
                    SELECT MAX(id)
                    FROM data_points
                    GROUP BY device_id
                )
            ) as last_data_point

            ON
            last_data_point.device_id = devices.id

            WHERE
                devices.id = ${deviceId}
        `);

        res.send(currentDevice);
        return;
    }

    const errorAgregator = new ErrorAgregator();

    if (req.body.name != null && req.body.name.trim() === "") {
        errorAgregator.pushError("name", "name can't be null or empty string");
    }

    if (req.body.update_frequency_seconds != null && (Number.isNaN(Number.parseInt(req.body.update_frequency_seconds)) || Number.parseInt(req.body.update_frequency_seconds) <= 0)) {
        errorAgregator.pushError("update_frequency_seconds", "update_frequency_seconds must be a number > 0");
    }

    if (req.body.min_threshold != null && (Number.isNaN(Number.parseInt(req.body.min_threshold)) || Number.parseInt(req.body.min_threshold) < 0)) {
        errorAgregator.pushError("min_threshold", "Minimum threshold must be a number >=0");
    }

    if (req.body.max_threshold != null && (Number.isNaN(Number.parseInt(req.body.max_threshold)) || Number.parseInt(req.body.max_threshold) > 100)) {
        errorAgregator.pushError("max_threshold", "Maximum threshold must be a number <=100");
    }

    if (
        (req.body.max_threshold !== null && req.body.max_threshold !== undefined || req.body.min_threshold !== null && req.body.min_threshold !== undefined)
        && Number.parseInt(req.body.min_threshold) > Number.parseInt(req.body.max_threshold)
    ) {
        errorAgregator.pushError("max_threshold", `Maximum threshold (${req.body.max_threshold}) must be greater than minimum threshold (${req.body.min_threshold})`);
    }

    if (errorAgregator.errors.length > 0) {
        res.status(400);
        res.send(errorAgregator.errors);
        return;
    }

    const fileUUID = crypto.randomUUID();
    const filePath = path.join(process.cwd(), `/static/img/${fileUUID}.jpeg`);

    if (req.body.base64img !== null && req.body.base64img !== undefined && req.body.base64img.trim() !== "") {
        try {
            const fileData = Buffer.from(req.body.base64img, "base64url");

            await fs.writeFile(filePath, fileData);
        }
        catch (exc) {
            res.send({
                error: "Error saving image"
            });
            console.log(exc);
            return;
        }
    }

    const formatedValues = [
        req.body.name ? `name = "${req.body.name}"` : undefined,
        req.body.update_frequency_seconds ? `update_frequency_seconds = ${req.body.update_frequency_seconds}` : undefined,
        req.body.min_threshold ? `min_threshold = ${req.body.min_threshold}` : undefined,
        req.body.max_threshold ? `max_threshold = ${req.body.max_threshold}` : undefined,
        req.body.base64img ? `pic_url = "/img/${fileUUID}.jpeg"` : undefined,
    ].filter(val => val).join(", \n");

    await db.query(`
        UPDATE
            devices
        
        SET
            ${formatedValues}

        WHERE
            devices.id = ${deviceId}
    `);

    const updatedDevice = await db.query(`
        SELECT 
        devices.id,
        devices.name, 
        devices.pic_url, 
        devices.min_threshold, 
        devices.max_threshold,
        last_data_point.point_transmited_datetime as "last_data_point_date",
        last_data_point.value as "last_data_point_value" 

        FROM 
            devices 
            
        LEFT JOIN (
            SELECT data_points.device_id, data_points.point_transmited_datetime, data_points.value
            FROM data_points
            WHERE id IN (
                SELECT MAX(id)
                FROM data_points
                GROUP BY device_id
            )
        ) as last_data_point

        ON
        last_data_point.device_id = devices.id

        WHERE
            devices.id = ${deviceId}
    `);

    res.send(updatedDevice[0]);
});

export default router;