import { useLocalSearchParams, router } from 'expo-router';
import { SafeAreaProvider, SafeAreaView } from 'react-native-safe-area-context';
import { Camera, CameraType, ImageType } from "expo-camera"
import { View, StyleSheet, Platform, ScrollView, ImageBackground, Pressable } from 'react-native';
import { Appbar, Text, ActivityIndicator, TextInput, Button, Snackbar, Modal, Icon } from 'react-native-paper';
import { useEffect, useRef, useState } from "react"
import axios from "axios"

export default function DeviceDetail() {
    const { id } = useLocalSearchParams();

    const device = useRef();
    const cameraReady = useRef(false);
    const imageInputRef = useRef();
    const imageBase64 = useRef();
    /** @type {React.MutableRefObject<Camera | undefined>} */
    const cameraRef = useRef();

    const [loading, setLoading] = useState(true);
    const [previewImageUrl, setPreviewImageUrl] = useState("");
    const [name, setName] = useState();
    const [minThreshold, setMinThreshold] = useState();
    const [maxThreshold, setMaxThreshold] = useState();
    const [updateFrequencySeconds, setUpdateFrequencySeconds] = useState();
    const [successSnackbarVisible, setSnackbarVisible] = useState(false);
    const [errorModalVisible, setErrorModalVisible] = useState(false);
    const [error, setError] = useState();
    const [cameraOpen, setCameraOpen] = useState(false);

    const [permission, requestPermission] = Camera.useCameraPermissions();

    function updateMinThreshold(textValue) {
        //regex goodnes XD
        const regex = /[0-9]{0,5}\.[0-9]{0,2}|^[0-9]{0,5}(?![0-9])/g;
        const regextResult = regex.exec(textValue);

        if (regextResult !== null && regextResult !== undefined && regextResult.length > 0) {
            setMinThreshold(regextResult[0])
        }
    }

    function updateMaxThreshold(textValue) {
        //regex goodnes XD
        const regex = /[0-9]{0,5}\.[0-9]{0,2}|^[0-9]{0,5}(?![0-9])/g;
        const regextResult = regex.exec(textValue);

        if (regextResult !== null && regextResult !== undefined && regextResult.length > 0) {
            setMaxThreshold(regextResult[0])
        }
    }

    function updateUpdateFrequencySeconds(textValue) {
        //regex goodnes XD
        const regex = /[0-9]{0,5}/g;
        const regextResult = regex.exec(textValue);

        if (regextResult !== null && regextResult !== undefined && regextResult.length > 0) {
            setUpdateFrequencySeconds(regextResult[0])
        }
    }

    async function submitChanges() {
        if (minThreshold >= maxThreshold) {
            setError("Minimum threshold must be smaller than Maximum threshold");
            setErrorModalVisible(true);
        }

        try {
            const result = await axios.patch(`http://${global.API_URL}/api/devices/${id}`, {
                name: name,
                min_threshold: minThreshold,
                max_threshold: maxThreshold,
                update_frequency_seconds: updateFrequencySeconds,
                base64img: imageBase64 !== "" ? imageBase64.current : undefined
            })

            if (result.status === 200) {
                setSnackbarVisible(true);
                setTimeout(() => setSnackbarVisible(false), 3000);
            }
        }
        catch (err) {
            setErrorModalVisible(true);
            setError(JSON.stringify(err, undefined, 2));
        }
    }

    function updateImage() {
        if (Platform.OS === "web") {
            imageInputRef.current.click();
        }
        else {
            requestPermission();
            setCameraOpen(true);
        }
    }

    async function takePicture() {
        if (cameraReady && cameraOpen) {
            if (cameraRef.current) {
                const pictureData = await cameraRef.current.takePictureAsync({
                    base64: true,
                    imageType: ImageType.jpg,
                    quality: 0.8
                });

                imageBase64.current = pictureData.base64;
                setPreviewImageUrl(pictureData.uri);
                setCameraOpen(false);
            }
        }
    }

    async function handleBrowserImageUpload(changeEvent) {
        const file = changeEvent.target.files[0];
        if (file) {
            const fileReader = new FileReader();
            fileReader.onload = (ev) => {
                setPreviewImageUrl(ev.target.result);
                imageBase64.current = ev.target.result.split(",")[1];
                console.log(ev.target.result.split(","));
            }
            fileReader.readAsDataURL(file);
        }
    }

    useEffect(() => {
        async function getDeviceById(id) {
            const result = await axios.get(`http://${global.API_URL}/api/devices/${id}`);
            await new Promise((resolve) => { setTimeout(resolve, 1000) });

            device.current = result.data;

            setName(result.data.name);
            setMinThreshold(result.data.min_threshold);
            setMaxThreshold(result.data.max_threshold);
            setUpdateFrequencySeconds(result.data.update_frequency_seconds);
            setPreviewImageUrl(`http://${global.API_URL}${result.data.pic_url}`);
            setLoading(false);
        }

        getDeviceById(id);
    }, [])

    return (
        <SafeAreaProvider style={{ backgroundColor: "rgba(234, 221, 255, 1)" }}>
            <SafeAreaView style={styles.container}>
                <Appbar.Header style={styles.header}>
                    <Appbar.BackAction onPress={() => { router.navigate(`/device/${id}`) }}></Appbar.BackAction>
                    <Appbar.Content title="Device settings" />
                </Appbar.Header>
                <ScrollView style={styles.contentContainer} contentContainerStyle={{ alignItems: "center", minHeight: "100%" }}>
                    {
                        loading === false ? (
                            <View style={{
                                justifyContent: 'flex-start',
                                alignItems: "stretch",
                                maxWidth: 1000,
                                width: "100%",
                                padding: 15,
                                paddingTop: 20,
                                paddingBottom: 20,
                            }}>
                                <View style={{ flex: 1, justifyContent: "center", alignItems: "center", minHeight: 350 }}>
                                    <Pressable onPress={updateImage} style={{ width: "100%", height: "100%" }}>
                                        <ImageBackground
                                            source={{
                                                uri: previewImageUrl
                                            }}
                                            resizeMode='cover'
                                            style={{
                                                borderWidth: 10,
                                                borderRadius: 10000,
                                                borderColor: "rgba(234, 221, 255, 1)",
                                                width: "100%",
                                                height: "100%",
                                                overflow: "hidden",
                                                justifyContent: "center",
                                                alignItems: "center"
                                            }}>
                                            <Icon source={"camera"} size={100} color='rgba(234, 221, 255, 1)'></Icon>
                                        </ImageBackground>

                                    </Pressable>
                                </View>
                                {Platform.OS === "web" && <input type='file' accept='.jpeg' ref={imageInputRef} style={{ display: "none" }} onChange={handleBrowserImageUpload} ></input>}
                                <View style={{ ...styles.inputGroup }}>
                                    <Text>Name:</Text>
                                    <TextInput value={name} inputMode={"text"} onChangeText={setName} maxLength={50}></TextInput>
                                </View>
                                <View style={{ ...styles.inputGroup }}>
                                    <Text>Minimum water threshold:</Text>
                                    <TextInput value={minThreshold.toString()} inputMode={"decimal"} onChangeText={updateMinThreshold}></TextInput>
                                </View>
                                <View style={{ ...styles.inputGroup }}>
                                    <Text>Maximum water threshold:</Text>
                                    <TextInput value={maxThreshold.toString()} inputMode={"decimal"} onChangeText={updateMaxThreshold}></TextInput>
                                </View>
                                <View style={{ ...styles.inputGroup }}>
                                    <Text>Update frequency (seconds):</Text>
                                    <TextInput value={updateFrequencySeconds.toString()} inputMode={"numeric"} onChangeText={updateUpdateFrequencySeconds}></TextInput>
                                </View>
                                <Button style={{ marginTop: 20, maxWidth: 120, alignSelf: "center" }} icon={"check"} mode={"contained"} onPress={submitChanges}>Save</Button>
                            </View>
                        ) : (
                            <View style={{ flex: 1, justifyContent: "center", alignItems: "center", maxWidth: 1000 }}>
                                <ActivityIndicator size={"large"}></ActivityIndicator>
                                <Text style={{ marginTop: 10 }} variant='titleLarge'>Loading...</Text>
                            </View>
                        )
                    }
                </ScrollView>
                <Snackbar visible={successSnackbarVisible}>Device settings saved</Snackbar>
                <Modal
                    visible={errorModalVisible}
                    onDismiss={() => setErrorModalVisible(false)}
                    contentContainerStyle={{ backgroundColor: 'white', padding: 20, maxWidth: 1000, alignSelf: "center" }}>
                    <Text>{error}</Text>
                </Modal>
            </SafeAreaView>
            {
                cameraOpen &&
                <View style={{ position: "absolute", width: "100%", height: "100%", backgroundColor: "black", justifyContent: "center" }}>
                    <Pressable style={{ position: "absolute", top: 10, right: 10, width: 50, height: 50, justifyContent: "center", alignItems: "center" }} onPress={() => { setCameraOpen(false); cameraReady.current = false }}>
                        <Icon source={"close"} size={40} color='white'></Icon>
                    </Pressable>
                    <Camera style={{ width: "100%", aspectRatio: "3/4" }} type={CameraType.back} onCameraReady={() => { cameraReady.current = true }} ref={cameraRef}>
                    </Camera>
                    <Pressable style={{ position: "absolute", bottom: 10, width: "100%", height: 100, justifyContent: "center", alignItems: "center" }} onPress={takePicture}>
                        <Icon source={"radiobox-marked"} size={80} color='white'></Icon>
                    </Pressable>
                </View>
            }
        </SafeAreaProvider >
    );
}

const styles = StyleSheet.create({
    container: {
        flex: 1,
        minHeight: "100%",
        backgroundColor: 'rgba(234, 221, 255, 1)',
    },
    header: {
        backgroundColor: 'rgba(234, 221, 255, 1)'
    },
    activityIndicatorContainer: {
        flex: 1,
        justifyContent: "center",
        alignContent: "center",
        backgroundColor: 'white'
    },
    contentContainer: {
        flex: 1,
        flexDirection: "column",
        backgroundColor: "white"
    },
    inputGroup: {
        justifyContent: "flex-start",
        alignItems: "stretch",
        marginTop: Platform.OS === "web" ? 30 : 10
    }
});